use strict;
use warnings;
package BenchmarkAnything::Storage::Frontend::Tools;
# git description: v0.018-2-g43fc70d

our $AUTHORITY = 'cpan:SCHWIGON';
# ABSTRACT: Access a BenchmarkAnything store with cmdline tools
$BenchmarkAnything::Storage::Frontend::Tools::VERSION = '0.019';
1;

__END__

=pod

=encoding UTF-8

=head1 NAME

BenchmarkAnything::Storage::Frontend::Tools - Access a BenchmarkAnything store with cmdline tools

=head1 AUTHOR

Steffen Schwigon <ss5@renormalist.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Steffen Schwigon.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
